package com.galaxis.manatee.service.impl;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.ChuanyunService;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author chongjiezhong
 */
@Slf4j
@Service
public class ChuanyunServiceImpl implements ChuanyunService {

    private final Integer pageSize=20;
    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;
    private final ChuanYunManager chuanYunManager;

    public ChuanyunServiceImpl(ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask, ChuanYunManager chuanYunManager) {
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
        this.chuanYunManager = chuanYunManager;
    }

    @Override
    public void updateProjectMember() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {

                    //处理项目对象中的项目成员数据(改为手动调用处理)
                    chuanyunBasicDataScheduledTask.projectMemberCheck(chuanyunGroupProjectDO);
                    //只处理项目成员为空的情况
                    boolean emptyFlag= chuanyunGroupProjectDO.getProjectMemberList()==null||chuanyunGroupProjectDO.getProjectMemberList().length==0;
                    if(emptyFlag){
                        GroupProjectUpdateDTO groupProjectUpdateDTO= new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
                        //生成更新数据后校验数据是否增加成功
                        if(groupProjectUpdateDTO.getProjectMemberList()!=null&&groupProjectUpdateDTO.getProjectMemberList().length!=0){
                            try {
                                var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                                ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                                if(chuanyunSaveDTO.getSuccessful()){
                                    log.info(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员成功");

                                }else{
                                    log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员失败");
                                    log.warn(chuanyunGroupProjectDO+"");
                                }
                            } catch (BigSizeException | JsonProcessingException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                });
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    static class GroupProjectUpdateDTO{

        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberList;

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectMemberList()){
                    this.projectMemberList =chuanyunGroupProjectDO.getProjectMemberList();
                }

            }
        }
    }
}
